LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL ;
USE ieee.std_logic_unsigned.ALL ;

-- The ReadDeligater is loacated in the Stratix
--				 
--Purpose of ReadDeligater
--Select ClusterFifo by address and connect to the Daughter Board Data Bus dbdb.  The dbdb makes the connection
--from ClusterFifo to avalon switch, via an intervening FIFO.
--There are 5 ClusterFifos for each chip, up to 4 chips per daughter card and up to 3 daughter cards per
--ladder.  The ReadDeligator round robin connects one of these 60 ClusterFifos at a time to the dbdb
--As soon as the connection is made the ClusterFifo
--pushes its data out.
--
--Operation of the ReadDeligater
--Advance ClusterFifo Module address counter until a a CFM responds with read_ready.  Pause address counter 
--advance and start CFM writing data on to the dbdb.  When the CFM empties restart address counter to scan for
--another CFM with read_ready true .. etc.

entity ReadDeligater is
generic(
MAX_FIFO_NUM: integer :=60
);
port (
	reset_n : in std_logic ;
	   
-- clocks
	clk 	: in std_logic ; -- dbdb clock
	
-- control of this module							
--	enable : in std_logic; -- turns this module on.  enable comes as a control instruction via daughter CSR
						
							
-- status of this module or at least the ClusterFifoModules under it's control
--	db_sending : out std_logic; -- high when any ClusterFifo module is sending data to the receiver
								-- this could be same as cf_read_enable, we will see.					
							
-- interface to the ClusterFifoModules
	cf_address : inout std_logic_vector(6 downto 0); --cluster module address
	cf_read_ready : in std_logic; -- 1 indicates an addressed ClusterFifo module is ready for outputing
	cf_read_enable : out std_logic; -- tells addressed ClusterFifo to connect
--new control line 
   cf_control  : in 	std_logic;
	nios_command : in std_logic_vector(6 downto 0) --cluster module address

) ;
end entity ReadDeligater ;

architecture a of ReadDeligater is
--sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
type stype is (WAIT_FOR,SCAN);
signal state : stype:=WAIT_FOR;
--signal read_enable : std_logic; -- cycles the cluster module address
signal address_cnt : std_logic_vector(6 downto 0); -- cycles the cluster module address
signal delay       : std_logic_vector(2 downto 0):="000"; -- cycles the cluster module address
signal stable_cnt       : std_logic_vector(4 downto 0):="00000"; -- cycles the cluster module address
signal buf       : std_logic; -- cycles the cluster module address
signal readed       : std_logic:='0'; -- cycles the cluster module address
--signal nuff_time : std_logic_vector(11 donto 0 ); --more than sufficient for 2048 deep FIFOs
--ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
begin

-------------------------------------------------------------------------------------------
sm0:process(clk, reset_n)
begin
	if (reset_n ='0') then
			   address_cnt<="0000000";
			state<=WAIT_FOR;
	elsif(rising_edge(clk))  then
	buf<=cf_read_ready;
	case state is
    when WAIT_FOR	 =>
--		address_cnt<=CONV_STD_LOGIC_VECTOR(47,7);
--       if CONV_INTEGER(nios_command)<5 and CONV_INTEGER(address_cnt)=47 then
        state<=SCAN;
        address_cnt<="0000000";
		readed<='0';
--readed<='0';
      -- end if;

    when SCAN       =>
       	if buf='0'  then
				stable_cnt<="00000";
				cf_read_enable<='0';
					if  address_cnt="0001011" and readed='1' then
			   			
	          			state<=WAIT_FOR;
			
					elsif readed='1' then
						address_cnt<=address_cnt+1;
						readed<='0';
					end if;
		else
			if CONV_INTEGER(stable_cnt)<6 then
					stable_cnt<=stable_cnt+1;
			else
					cf_read_enable<='1';
					readed<='1';
			end if;
		end if;



	when NULL       =>
	state<=WAIT_FOR;
	end case;

	end if;
end process;
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- sm0:process(clk, reset_n)
-- begin
	-- if (reset_n ='0') then
			   -- address_cnt<="0000000";
			-- state<=WAIT_FOR;
	-- elsif(rising_edge(clk))  then
	-- buf<=cf_read_ready;
	-- case state is
    -- when WAIT_FOR	 =>
		-- address_cnt<=CONV_STD_LOGIC_VECTOR(47,7);
       -- if CONV_INTEGER(nios_command)<5 and CONV_INTEGER(address_cnt)=47 then
        -- state<=SCAN;
        -- address_cnt<="0000000";
       -- end if;

    -- when SCAN       =>
       	-- if buf='0'  then
				-- cf_read_enable<='0';
				-- delay<=delay+1;
				-- if delay="111" then
					-- if  address_cnt="0000010" then
			   			
	          			-- state<=WAIT_FOR;
			
					-- else
						-- address_cnt<=address_cnt+1;
					-- end if;
				-- end if;
			-- else
				-- cf_read_enable<='1';
				-- delay<="000";
		-- end if;



	-- when NULL       =>
	-- state<=WAIT_FOR;
	-- end case;

	-- end if;
-- end process;
----------------------------------------------------------------------------------------------
--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
	 cf_address <= address_cnt;
--	cf_read_enable<='1' when  cf_read_ready='1' and cf_control='0' else '0';
--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
end architecture;