LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL ;
USE ieee.std_logic_unsigned.ALL ;



ENTITY chinhvu IS
	PORT
	(
--	Dev. Kit. ******************
		--		resets
		RHIC_CLK			: in  STD_LOGIC ;-- read request
		CLK_CONTROL			: out  STD_LOGIC ;-- read request
		fifo_read_REQ			: out  STD_LOGIC ;-- read request
		fifo_read_ready			: in  STD_LOGIC ;-- read request

		fifo_data1			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data2			: INout	STD_LOGIC_VECTOR(19 downto 0) ;-- the data I.E. the address of cluster center
		fifo_data3			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data4			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data5			: INout	STD_LOGIC_VECTOR(8 downto 0) ;-- the data I.E. the address of cluster center

		fifo_addr			: INout	STD_LOGIC_VECTOR(6 downto 0) ;-- the data I.E. the address of cluster center

		out_trg_come 		: out std_logic;
		out_reset_n 		: out std_logic;
		out_clock 		: out std_logic;
--ddl
		fbD			: INOUT	STD_LOGIC_VECTOR(31 downto 0) ;
		fbCTRL_N	: INOUT	STD_LOGIC ;
		fbTEN_N		: INOUT	STD_LOGIC ;

		fiDIR		: IN STD_LOGIC ;
		fiBEN_N		: IN STD_LOGIC ;
		fiLF_N		: IN STD_LOGIC ;

		foBSY_N		: OUT STD_LOGIC ;
		foCLK		: OUT STD_LOGIC;
		com		: OUT STD_LOGIC;
		test : out  std_logic_vector(6 downto 0) ;		
--avalon
		ava_clk : in std_logic ;
		ava_cs : in std_logic ;

		ava_wdata : in std_logic_vector(31 downto 0) ;
		ava_rdata : out std_logic_vector(31 downto 0) ;
			
		ava_write_n : in std_logic ;
		ava_addr : in std_logic_vector(5 downto 0);
	 
				
--
		TMS_out : out std_logic;
		TDI_out : out std_logic;
		TCK_out : out std_logic;
		TDO_in 	: in std_logic;
		RSTB_jtag: out std_logic;
		Cluster_FIFO_Data_Valid 	: in std_logic
	);
END chinhvu;

ARCHITECTURE a OF chinhvu IS
--cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
component myfifo
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		wrreq		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		rdclk		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		aclr		: IN STD_LOGIC  := '0';
		q		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC ;
		wrusedw		: OUT STD_LOGIC_VECTOR (13 DOWNTO 0)
	);
end component;
component readdeligater
	generic (
				MAX_FIFO_NUM : integer := 60
				);
port (
	reset_n : in std_logic ;
	   
	clk 	: in std_logic ; -- dbdb clock
	cf_address : inout std_logic_vector(6 downto 0); --cluster module address
	cf_read_ready : in std_logic; -- 1 indicates an addressed ClusterFifo module is ready for outputing
	cf_read_enable : out std_logic; -- tells addressed ClusterFifo to connect
--new control line 
   cf_control  : in 	std_logic;
	nios_command : in std_logic_vector(6 downto 0) --cluster module address
) ;
end component;
component infifo
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		wrreq		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		rdclk		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		aclr		: IN STD_LOGIC  := '0';
		q		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;
component framefifo
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		wrreq		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		rdclk		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		aclr		: IN STD_LOGIC  := '0';
		q		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;

component jtag_programmer_avalon 
	Generic (
		JTAG_IR_size 		: integer := 5;
		Data_Length_Bits	: integer := 10;
		Clock_Divider_Bits 	: integer := 8 -- 2,3,4,5...
		);
	Port ( 
		--Avalon signals
		avalon_clk 			: in std_logic;						
		avalon_reset 		: in std_logic;						
		avalon_chipselect 	: in std_logic;						
		avalon_address 		: in std_logic_vector(1 downto 0); 	
		avalon_write 		: in std_logic;						
		avalon_writedata 	: in std_logic_vector(31 downto 0); 
		avalon_read 		:  in std_logic;					
		avalon_readdata 	: out std_logic_vector(31 downto 0);
		
		-- JTAG outputs
		TMS_out : out std_logic;
		TDI_out : out std_logic;
		TCK_out : out std_logic;
		TDO_in 	: in std_logic;
		RSTB_out: out std_logic
		);
end component;
--cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
--ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
signal		avalon_clk 			:  std_logic;						
signal		avalon_reset 		:  std_logic;						
signal		avalon_chipselect 	:  std_logic;						
signal		avalon_address 		:  std_logic_vector(1 downto 0); 	
signal		avalon_write 		:  std_logic;						
signal		avalon_writedata 	:  std_logic_vector(31 downto 0); 
signal		avalon_read 		:   std_logic;					
signal		avalon_readdata 	:  std_logic_vector(31 downto 0);
signal		avalon_readdata_back 	:  std_logic_vector(31 downto 0);
		
		-- JTAG outputs


signal		reset_n		:  STD_LOGIC:='0';

signal		wrreq		:  STD_LOGIC ;
signal		wrreq_ddl		:  STD_LOGIC :='0';
signal		wrreq_bus	:  STD_LOGIC ;
signal		rdreq		:  STD_LOGIC ;
signal		rdclk		:  STD_LOGIC ;
signal		wrclk		:  STD_LOGIC ;
signal		aclr		:  STD_LOGIC  := '0';
signal		q			:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		ddl_q			:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		rdempty	:  STD_LOGIC ;
signal		wrfull	:  STD_LOGIC ;


signal		inwrreq		:  STD_LOGIC ;
signal		inrdreq		:  STD_LOGIC ;
signal		inaclr		:  STD_LOGIC  := '0';

signal		indata_in	:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		inq			:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		inrdempty	:  STD_LOGIC ;
signal		inwrfull	:  STD_LOGIC ;


signal		fifo_read_ENA	:  STD_LOGIC ;
signal		req_reg	:  STD_LOGIC ;
--signal		command	:  STD_LOGIC ;
signal		command	:  STD_LOGIC :='0';
signal		command_ddl	:  STD_LOGIC_VECTOR (1 DOWNTO 0):="00";
signal		command_addr	:  STD_LOGIC :='0';
signal		open_command	:  STD_LOGIC :='0';
signal		close_command	:  STD_LOGIC :='1';

signal		send_data	:  STD_LOGIC_VECTOR (6 DOWNTO 0);
signal		send_addr	:  STD_LOGIC_VECTOR (6 DOWNTO 0):="0000000";
signal		re_addr		:  STD_LOGIC_VECTOR (6 DOWNTO 0);
signal		states		:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		comd_in		:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		readflag		:  STD_LOGIC:='0';
signal		ddlreadflag		:  STD_LOGIC:='0';
signal		ddlcomtrig		:  STD_LOGIC:='0';


signal		mfbD			: 	STD_LOGIC_VECTOR(31 downto 0) ;
signal		mfbCTRL_N	: 	STD_LOGIC :='1';
signal		mfbTEN_N		: 	STD_LOGIC ;
signal		mfiDIR		:  STD_LOGIC ;
signal		mfiBEN_N		:  STD_LOGIC ;
signal		mfiLF_N		:  STD_LOGIC ;
signal		mfoBSY_N		:  STD_LOGIC ;
signal		mfoCLK		:  STD_LOGIC;
signal		mcom		:  STD_LOGIC;

signal		data_in	:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		data_in_ddl	:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		data_in_bus	:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		fifoused		:  STD_LOGIC_VECTOR (13 DOWNTO 0);
signal		show_fifoused		:  STD_LOGIC_VECTOR (13 DOWNTO 0);
signal		rorccmd		:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		rorccounter		: STD_LOGIC_VECTOR (31 DOWNTO 0):=(others=>'0');
signal		rorcdata		: STD_LOGIC_VECTOR (31 DOWNTO 0):=(others=>'0');
signal		rorcdataA		: STD_LOGIC_VECTOR (15 DOWNTO 0):=(others=>'0');
signal		waitcounter		: STD_LOGIC_VECTOR (9 DOWNTO 0):=(others=>'0');
signal		jtagcounter		: STD_LOGIC_VECTOR (31 DOWNTO 0):=(others=>'0');
signal		statecounter		: STD_LOGIC_VECTOR (3 DOWNTO 0):=(others=>'0');
signal		niosstatecounter		: STD_LOGIC_VECTOR (3 DOWNTO 0):=(others=>'0');
signal		eventcounter		: STD_LOGIC_VECTOR (32 DOWNTO 0):=(others=>'0');
signal		normal_counter		: STD_LOGIC_VECTOR (13 DOWNTO 0):=(others=>'0');
signal		eventstart		: STD_LOGIC:='0';
signal		timescounter		: STD_LOGIC_VECTOR (23 DOWNTO 0):=(others=>'0');
signal		block_data		: STD_LOGIC_VECTOR (31 DOWNTO 0):=(others=>'0');
signal		ddl_sent_data		: STD_LOGIC_VECTOR (31 DOWNTO 0):=(others=>'0');
signal		ddl_reset		: STD_LOGIC:='0';
signal		ddl_clk		: STD_LOGIC:='0';
signal		trg_come		: STD_LOGIC:='0';
signal		nios_req		: STD_LOGIC:='0';
signal		ddl_end		: STD_LOGIC:='0';


signal		frame_wrreq		: STD_LOGIC:='0';
signal		frame_rdreq		: STD_LOGIC:='0';
signal		frame_buffer_rdreq		: STD_LOGIC:='0';
signal		frame_q		:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		frame_buffer_q		:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		frame_buffer_q1		:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		buffer_q1		:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		buffer_q		:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		frame_data		:  STD_LOGIC_VECTOR (31 DOWNTO 0);
signal		frame_rdempty		: STD_LOGIC:='0';
signal		frame_wrfull		: STD_LOGIC:='0';
signal		control_clk		: STD_LOGIC:='0';
signal		control_fifo		: STD_LOGIC:='0';
signal		frame_fifo_read_ready		: STD_LOGIC:='0';
signal		gate		: STD_LOGIC:='0';
signal		wrreqdelay1		: STD_LOGIC:='0';
signal		wrreqdelay2		: STD_LOGIC:='0';
signal		endgate		: STD_LOGIC:='0';
signal		startgate		: STD_LOGIC:='0';
signal		my_req_counter		:  STD_LOGIC_VECTOR (3 DOWNTO 0):=(others=>'0');

--signal		RHIC_CLK		: STD_LOGIC:='0';

signal		data_num		:  STD_LOGIC_VECTOR (31 DOWNTO 0):=x"00014a00";
signal		event_num		:  STD_LOGIC_VECTOR (15 DOWNTO 0):=x"0008";
signal		data_control		:  STD_LOGIC_VECTOR (7 DOWNTO 0):=x"00";
signal		fake_counter		:  STD_LOGIC_VECTOR (31 DOWNTO 0):=(others=>'0');
signal		req_counter		:  STD_LOGIC_VECTOR (3 DOWNTO 0):=(others=>'0');
signal		reset_counter		:  STD_LOGIC_VECTOR (3 DOWNTO 0):=(others=>'0');
signal		trigger_counter		:  STD_LOGIC_VECTOR (3 DOWNTO 0):=(others=>'0');


type ddl_type is (WAIT_TRG, WAIT_BLOCK_BACK,WAIT_BLOCK, WAIT_EVENT,WAIT_EVENT_BACK,WAIT_EVENT_FRAME,WAIT_EVENT_BACK_FRAME,BLOCK_WRITE,END_BLOCK,EVENT_WRITE,END_EVENT,EVENT_WRITE_FRAME,END_EVENT_FRAME,BLOCK_WRITE_FROM_RORC);
signal state: ddl_type:=WAIT_TRG;
type nios_type is (WAIT_COMMAND,GET_COMMAND,DISPATCH_COMMAND,EXE_COMMAND,EXE_READ,EXE_WRITE,EXE_RESET,EXE_TRIGGER,EXE_COMMAND_ADDR,EXE_ADDR,EXE_DATA_A,EXE_DATA_B,EXE_DATA_CONTROL,AVA_CS_JTAG,AVA_ADDR_JTAG,AVA_DATA_A_JTAG,AVA_DATA_B_JTAG,AVA_WAIT_JTAG);
signal niosstate: nios_type:=WAIT_COMMAND;

signal avalon_buffer: std_logic_vector(31 downto 0);


--ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
BEGIN

--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww

out_clock<=ava_clk;

--out_trg_come<=trg_come;

out_reset_n<=reset_n;

CLK_CONTROL<=control_clk;

--RHIC_CLK<=ddl_clk;
-------------------------------------------------------------------------ddl state machine
process(ava_clk)
begin
if ava_clk='1' and ava_clk'event then
    if trg_come='1' and CONV_INTEGER(trigger_counter)=2 then
		trigger_counter<=(others=>'0');
		out_trg_come<='0';
	elsif CONV_INTEGER(trigger_counter)<2 then
		trigger_counter<=trigger_counter+1;
		out_trg_come<='1';
	else
		out_trg_come<='0';
	end if;	

	ddl_clk<=not ddl_clk;
--	frame_buffer_q<=frame_q;
--	frame_buffer_q<=fake_counter;
--	frame_buffer_rdreq<=frame_rdreq;
end if;
end process;
process(ava_clk)
begin
if ava_clk='0' and ava_clk'event then
--	ddl_clk<=not ddl_clk;
	frame_buffer_q1<=mfbD;
--	frame_buffer_q<=fake_counter;
--	frame_buffer_rdreq<=frame_rdreq;
end if;
end process;
foCLK<= ava_clk;

foBSY_N<= not inwrfull;


process (ava_clk,ddl_reset)
begin

if  ddl_reset='1' then
	state<=WAIT_TRG;
elsif ava_clk='1' and ava_clk'event then
	comd_in<=fbD;  						-- is comd_in used anywhere??
	ddl_q<=q;
	if mcom='1' then
		mcom<='0';
	elsif  fiDIR='0' and fbCTRL_N='0' and fbTEN_N='0' and fbD(7 downto 0)="10110100"  then
		ddl_end<='1';				
	end if;	

case state is

    when WAIT_TRG	 =>	
	statecounter<="0000";
	mfbCTRL_N<='1';
	mfbTEN_N<='1';
	
	
	if ava_cs='1' and  ava_addr="000100" and command='0' and control_fifo='1' then
		frame_rdreq<='1';
	else
		frame_rdreq<='0';
	end if;
		
	if  fiDIR='0' and fbCTRL_N='0' and fbTEN_N='0' and fbD(7 downto 0)="01010100"  then
		waitcounter<=(others=>'0');
		state<=WAIT_BLOCK;
		mcom<='1';
	elsif ava_cs='1' and  ava_addr="000100" and command='0' and control_fifo='0' then
		rdreq<='1';
	else
		rdreq<='0';
	end if;


	if  fiDIR='0' and fbCTRL_N='0' and fbTEN_N='0' and fbD(7 downto 0)="11010100"  then
		waitcounter<=(others=>'0');
		state<=BLOCK_WRITE_FROM_RORC;
	end if;		
		
	if  fiDIR='0' and fbCTRL_N='0' and fbTEN_N='0' and fbD(7 downto 0)="00010100"   then
		waitcounter<=(others=>'0');
		eventcounter<=(others=>'0');
		req_counter<=(others=>'0');
		eventstart<='1';
		state<=WAIT_EVENT;
		ddl_end<='0';
		timescounter<=x"000018";
		mcom<='1';
		my_req_counter<="0001";
	else
		eventstart<='0';
	end if;	
		
--if timescounter/=x"000000" then
--timescounter<=timescounter-1;
--				waitcounter<=(others=>'0');
--				state<=WAIT_EVENT;
--				mcom<='1';
--end if;
			
--		null;	
	when WAIT_BLOCK=>
	mfbTEN_N<='1';
	if CONV_INTEGER(waitcounter)>6 then
		waitcounter<=(others=>'0');
		state<=BLOCK_WRITE;
		rorccounter<=(others=>'0');
	else
		waitcounter<=waitcounter+1;
	end if;

	statecounter<="0001";
	null;
	
	when BLOCK_WRITE=>
	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0'  then
		
--				if rorccounter=x"0000100" then
		if rdempty='1' or CONV_INTEGER(rorccounter)>16000 then
			mfbCTRL_N<='1';
			mfbTEN_N<='0';
			--mfbD<=rorccounter;				
			mfbD<=x"00000000";
			rorccounter<=rorccounter+1;
			rdreq<='1';
		
			rorccounter<=(others=>'0');
			waitcounter<=(others=>'0');
			state<=WAIT_BLOCK_BACK;
		else
			mfbCTRL_N<='1';
			if rdreq='1' then
			mfbTEN_N<='0';
			else
			mfbTEN_N<='1';
			end if;
			--mfbD<=rorccounter;				
			mfbD<=q;
			rdreq<='1';
		end if;
	else
		mfbTEN_N<='1';
		rdreq<='0';
	end if;
	rorccounter<=rorccounter+1;
	statecounter<="0010";
	null;
	
	when WAIT_BLOCK_BACK=>
	mfbTEN_N<='1';
	rdreq<='0';
	if waitcounter="0000001100" then
		waitcounter<=(others=>'0');
		state<=END_BLOCK;
		rorccounter<=(others=>'0');
	else
		waitcounter<=waitcounter+1;
	end if;

	statecounter<="0011";
	null;
	
	when END_BLOCK=>
	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0'   then
		mfbCTRL_N<='0';
		mfbTEN_N<='0';
		mfbD<="00000000000000000000000001100100";				
		state<=WAIT_TRG;
	else
		mfbTEN_N<='1';	
	end if;	
	statecounter<="0100";
	null;
	
	when WAIT_EVENT=>

	mfbTEN_N<='1';
	if ddl_end='1' or CONV_INTEGER(eventcounter)>=CONV_INTEGER(event_num) then 

		state<=WAIT_TRG;
	elsif rdempty='0' and control_fifo='0'  then
					
	
		if waitcounter="0000001111" then
			waitcounter<=(others=>'0');
			state<=EVENT_WRITE;
			rorccounter<=(others=>'0');
		else
			waitcounter<=waitcounter+1;
		end if;
	elsif frame_rdempty='0' and control_fifo='1'  then
		if waitcounter="0000011111" then
			waitcounter<=(others=>'0');
			state<=EVENT_WRITE;
			rorccounter<=(others=>'0');
			
		else
			waitcounter<=waitcounter+1;
		end if;
	end if;
	statecounter<="0101";
	null;
	

	when EVENT_WRITE=>

--			 if buffer_q(31 downto 28)=x"e"  then
--					 gate<='0';
--			else	
--			 if q(27 downto 24)=x"a" and rdempty='0' then
--					aaaa_counter<="00000000";
--					 gate<='1';
--					startgate<='1';
--			else
					
--					startgate<='0';
--			 end if;
--			end if;
			
--			if buffer_q(27 downto 24)=x"5" and gate='1' then
--					 endgate<='1';
--			 end if;	

--	if  endgate='1' and gate='1' then
--				gate<='0';
--	end if;
	
	
	
			
	if ddl_end='1' then 
		state<=WAIT_EVENT_BACK;
	end if;
	
	if q(31 downto 28)=x"5" and CONV_INTEGER(rorccounter)>10 then
	gate<='1';
	else
	gate<='0';
	end if;

--	if q(27 downto 24)=x"a"  then
--	startgate<='1';
--	else
--	startgate<='0';
--	end if;			 


	
	
	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0' and control_fifo='0' and gate='1' then
			
			
				mfbTEN_N<='1';
				rorccounter<=(others=>'0');
				waitcounter<=(others=>'0');
				rdreq<='0';
				if rdempty='0' then
					my_req_counter<=(others=>'0');
					buffer_q<=q;
					--q<=buffer_q1;

				end if;
				state<=WAIT_EVENT_BACK;
	end if;


	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0' and control_fifo='0' and gate='0' and rdempty='0' and CONV_INTEGER(my_req_counter)>0 then
							mfbCTRL_N<='1';
							if rdreq='1' then
								mfbTEN_N<='0';
								fake_counter<=fake_counter+1;
								mfbD<=q ;
								rorccounter<=rorccounter+1;

								--q<=buffer_q1;
								
								--if	fifo_read_ready='0' then
								--mfbD<=x"55555555" ;
								--fifo_read_ready<='1';	
								--else
								--mfbD<=x"aaaaaaaa" ;	
								--fifo_read_ready<='0';
								--end if;
							else
								mfbTEN_N<='1';
							end if;
							rdreq<='1';
	end if;
	
	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0' and control_fifo='0' and gate='0' and rdempty='0' and CONV_INTEGER(my_req_counter)=0 then
							my_req_counter<=my_req_counter+1;
							mfbTEN_N<='0';
							mfbD<=buffer_q ;	
							mfbCTRL_N<='1';
							rorccounter<=rorccounter+1;
							rdreq<='0';
	end if;
	
	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0' and control_fifo='0' and gate='0' and rdempty='1'  then
					mfbTEN_N<='1';
					rdreq<='0';
	end if;	
	
	
	
	
	
	
	












	
	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0' and control_fifo='1' and CONV_INTEGER(rorccounter)>=CONV_INTEGER(data_num) then
			
				eventcounter<=eventcounter+1;
				mfbTEN_N<='1';
				rorccounter<=(others=>'0');
				waitcounter<=(others=>'0');
				frame_rdreq<='0';
				if frame_rdempty='0' then
					req_counter<=(others=>'0');
					frame_buffer_q<=frame_q;
					--frame_q<=frame_buffer_q1;

				end if;
				state<=WAIT_EVENT_BACK;
	end if;


	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0' and control_fifo='1' and CONV_INTEGER(rorccounter)<CONV_INTEGER(data_num) and frame_rdempty='0' and CONV_INTEGER(req_counter)>0 then
							mfbCTRL_N<='1';
							if frame_rdreq='1' then
								mfbTEN_N<='0';
								fake_counter<=fake_counter+1;
								mfbD<=frame_q ;
								rorccounter<=rorccounter+1;

								--frame_q<=frame_buffer_q1;
								
								--if	frame_fifo_read_ready='0' then
								--mfbD<=x"55555555" ;
								--frame_fifo_read_ready<='1';	
								--else
								--mfbD<=x"aaaaaaaa" ;	
								--frame_fifo_read_ready<='0';
								--end if;
							else
								mfbTEN_N<='1';
							end if;
							frame_rdreq<='1';
	end if;
	
	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0' and control_fifo='1' and CONV_INTEGER(rorccounter)<CONV_INTEGER(data_num) and frame_rdempty='0' and CONV_INTEGER(req_counter)=0 then
							req_counter<=req_counter+1;
							mfbTEN_N<='0';
							mfbD<=frame_buffer_q ;	
							mfbCTRL_N<='1';
							rorccounter<=rorccounter+1;
							frame_rdreq<='0';
	end if;
	
	if  fiDIR='1' and fiLF_N='1' and fiBEN_N='0' and control_fifo='1' and CONV_INTEGER(rorccounter)<CONV_INTEGER(data_num) and frame_rdempty='1'  then
					mfbTEN_N<='1';
					frame_rdreq<='0';
	end if;
	
	if  fiDIR='0' or fiLF_N='0' or fiBEN_N='1' then
			mfbTEN_N<='1';
		rdreq<='0';
		frame_rdreq<='0';
		if	CONV_INTEGER(req_counter)>0 and frame_rdempty='0' and control_fifo='1' then
			frame_buffer_q<=frame_q;
--			frame_q<=frame_buffer_q1;

			req_counter<=req_counter-1;
		end if;
			if	CONV_INTEGER(my_req_counter)>0 and rdempty='0' and control_fifo='0' then
			buffer_q<=q;
			my_req_counter<=my_req_counter-1;
		end if;
	end if;
	
	

	statecounter<="0110";
	


	when WAIT_EVENT_BACK=>
		mfbTEN_N<='1';

	if waitcounter="0000001111" then
		waitcounter<=(others=>'0');
		state<=END_EVENT;
		rorccounter<=(others=>'0');
	else
		waitcounter<=waitcounter+1;
	end if;
	
	statecounter<="0111";


	when END_EVENT=>
	if ddl_end='1'  then 
		mfbCTRL_N<='0';
		mfbTEN_N<='0';
		mfbD<="00000000000000000000000001100100";	
		state<=WAIT_TRG;
		elsif  fiDIR='1' and fiLF_N='1' and fiBEN_N='0'   then
		mfbCTRL_N<='0';
		mfbTEN_N<='0';
		mfbD<="00000000000000000000000001100100";				
		state<=WAIT_EVENT;
	else
		mfbTEN_N<='1';	
	end if;	
	null;
	statecounter<="1000";

	when BLOCK_WRITE_FROM_RORC=>
	if  fiDIR='0' and fbCTRL_N='1' and fbTEN_N='0'  and fiLF_N='1' and fiBEN_N='0' then
		indata_in<=fbD;
		inwrreq<='1';
	else
		inwrreq<='0';
	end if;	
	if  fiDIR='0' and fbCTRL_N='0' and fbTEN_N='0' and fbD(7 downto 0)="10110100"   then
		state<=WAIT_TRG;
	end if;	
		statecounter<="1001";
	
	when others=>
	state<=WAIT_TRG;
			statecounter<="1110";
end case;

	
end if;
end process;

com<=mcom;

fbD<=(others=>'Z') when  fiBEN_N='1'  else
 frame_buffer_q1 when  fiDIR='1'    else
 (others=>'Z');
fbTEN_N<='Z' when  fiBEN_N='1'  else
mfbTEN_N when  fiDIR='1'  else
 'Z';
fbCTRL_N<='Z' when  fiBEN_N='1'  else
mfbCTRL_N when fiDIR='1'  else
 'Z';



--test(1)<=mfbTEN_N;

--test(0)<=mfbCTRL_N;
--test(2)<=ddl_clk;
--test(4 downto 3)<=mfbD(1 downto 0);
--test(5)<=fiDIR;
--test(6)<=fiLF_N;

-------------------------------------------------------------------------ddl
--test(6 downto 4)<=avalon_chipselect & avalon_read & avalon_write;
--test(6 downto 4)<=avalon_address(1 downto 0)&avalon_write;
-------------------------------------------------------------------------nios
process (ava_clk,ddl_reset)
begin

if  ddl_reset='1' then
	state<=WAIT_TRG;
elsif ava_clk='0' and ava_clk'event then
	--test(1 downto 0)<=avalon_address;
	--test(3 downto 2)<=avalon_writedata(1 downto 0);
	
--avalon_writedata<=avalon_buffer;
	--test(2 downto 0)<=avalon_readdata_back(2 downto 0); --niosstatecounter;
	--test(3 downto 0)<=niosstatecounter;
	--test(6 downto 4)<=avalon_readdata(2 downto 0);
	test(0)<=not fiLF_N;
	if reset_n='0' then
		send_addr<="0000010";
		command<='0';
		command_addr<='0';
	end if;
	
	case niosstate is

    when WAIT_COMMAND	 =>
	jtagcounter<=(others=>'0');
--rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr	
	-- if ava_cs='1' and ava_write_n='0' then
		-- case ava_addr is
			-- when "000001" =>
				-- command <= ava_wdata(0);
				-- control_clk <= ava_wdata(1);
				-- control_fifo <= ava_wdata(2);
			-- when "000000" =>
				-- command_addr <= ava_wdata(0);
			-- when "001000" =>
				-- if ava_wdata(1 downto 0)="10" then
					-- trg_come<='1';
					-- reset_n<='1';
				-- elsif ava_wdata(1 downto 0)="01" then
					-- trg_come<='0';
					-- reset_n<='0';
				-- else
					-- trg_come<='0';
					-- reset_n<='1';
				-- end if;
			-- when others =>
				-- null;
		-- end case;
	-- end if;
	
	if (ava_wdata(0)='0' and ava_addr="000001" and ava_cs='1' and ava_write_n='0') then
		command<='0'; 
	elsif ava_wdata(0)='1' and ava_addr="000001" and ava_cs='1' and ava_write_n='0' then
		command<='1';
	end if; 

	if (ava_wdata(1)='0' and ava_addr="000001" and ava_cs='1' and ava_write_n='0') then
		control_clk<='0'; 
	elsif ava_wdata(1)='1' and ava_addr="000001" and ava_cs='1' and ava_write_n='0' then
		control_clk<='1';
	end if; 

	if (ava_wdata(2)='0' and ava_addr="000001" and ava_cs='1' and ava_write_n='0') then
		control_fifo<='0'; 
	elsif ava_wdata(2)='1' and ava_addr="000001" and ava_cs='1' and ava_write_n='0' then
		control_fifo<='1';
	end if; 

	if (ava_wdata(0)='0' and ava_addr="000000" and ava_cs='1' and ava_write_n='0') then
		command_addr<='0';
	elsif ava_wdata(0)='1' and ava_addr="000000" and ava_cs='1' and ava_write_n='0' then
		command_addr<='1';
	end if;
--rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
	if ava_addr="000010" and ava_cs='1' then
		send_addr<=ava_wdata(6 downto 0);
	end if;
	
--rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
	if ava_addr="001000"  and ava_cs='1' and ava_write_n='0' and ava_wdata(1 downto 0)="10" then
		trg_come<='1';
	else
		trg_come<='0';
	end if;

	if ava_addr="001000"  and ava_cs='1' and ava_write_n='0' and ava_wdata(1 downto 0)="01" then
		reset_n<='0';
	else
		reset_n<='1';
	end if;
--rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr

	if ava_addr="001001" and ava_cs='1' and ava_write_n='1' then
		inrdreq<='1';
	else
		inrdreq<='0';
	end if;
			
	if ava_addr="000100" and ava_cs='1' and ava_write_n='0' and command='1' then
		fifo_data1<=ava_wdata(31);
		fifo_data2<=ava_wdata(30 downto 11);
		fifo_data3<=ava_wdata(10);
		fifo_data4<=ava_wdata(9);
		fifo_data5<=ava_wdata(8 downto 0);
	else
		fifo_data1<='Z';
		fifo_data2<=(others=>'Z');
		fifo_data3<='Z';
		fifo_data4<='Z';
		fifo_data5<=(others=>'Z');
	end if;
	
	
	if command='0' and ava_cs='1' and ava_addr="000100" and control_fifo='0' then
		--ava_rdata<=q;
		ava_rdata<=ddl_q;
	end if;
	
	if command='0' and ava_cs='1' and ava_addr="000100" and control_fifo='1' then
		ava_rdata<=frame_q;
	end if;
	
	if command='1' and ava_cs='1' and ava_addr="000100" and ava_write_n='1' then
		ava_rdata<=data_in(31 downto 0);
--			ava_rdata<=x"00000008";
	end if;
--rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr	
	-- if ava_cs='1' and ava_write_n='1' then
		-- case ava_addr is
		-- when "000101" => ava_rdata<=states;
		-- when "000111" => ava_rdata<=block_data;
		-- when "001001" => ava_rdata<=inq;
		-- when others => null;
		-- end case;
	-- end if;
	if ava_cs='1' and ava_addr="000101" and ava_write_n='1' then
		ava_rdata<=states;
	end if;
			
	if ava_addr="000111" and ava_cs='1' and ava_write_n='1' then
		ava_rdata<=block_data;
	end if;
		
	if ava_addr="001001" and ava_cs='1' and ava_write_n='1' then
		ava_rdata<=inq;
	end if;
	
	if ava_addr="001010" and ava_cs='1' and ava_write_n='0' then
		data_control<=ava_wdata(7 downto 0);
	end if;
	
	if ava_addr="001011" and ava_cs='1' and ava_write_n='0' then
		if data_control=x"01" then
		data_num<=ava_wdata;
		end if;
		if data_control=x"02" then
		event_num<=ava_wdata(15 downto 0);
		end if;
	end if;
	
	
--rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr			
			
--			ava_rdata<=q 		when command='0' and ava_cs='1' and ava_addr="000100" else
--          data_in(31 downto 0)  when command='1' and ava_cs='1' and ava_addr="000100" and ava_write_n='1' else
--			states					when	ava_cs='1' and ava_addr="000101" and ava_write_n='1' else
--			block_data				when 	ava_addr="000111" and ava_cs='1' and ava_write_n='1' else
--			inq					when ava_addr="001001" and ava_cs='1' and ava_write_n='1';

	if inrdempty='0' then
		niosstate<=GET_COMMAND;
		inrdreq<='1';
	end if;
	niosstatecounter<=x"F";
    
	when GET_COMMAND	 =>	
	inrdreq<='0';
	rorcdata<=inq;
	niosstate<=DISPATCH_COMMAND;
	niosstatecounter<=x"E";

	when DISPATCH_COMMAND	 =>	
	case rorcdata(19 downto 16) is
		when x"0"=>
		niosstate<=EXE_COMMAND;
		
		when x"1"=>
		niosstate<=EXE_COMMAND_ADDR;		
		
		when x"2"=>
		niosstate<=EXE_ADDR;		
			
		when x"3"=>
		niosstate<=EXE_DATA_A;		
			
		when x"4"=>
		niosstate<=EXE_DATA_B;
			
		when x"5"=>
		niosstate<=EXE_RESET;
	
		when x"6"=>
		niosstate<=EXE_TRIGGER;
			
		when x"7"=>
		niosstate<=EXE_READ; -- EXE_WRITE is missing
			
		when x"8"=>
		niosstate<=AVA_CS_JTAG;
			
		when x"9"=>
		niosstate<=AVA_ADDR_JTAG;
			
		when x"a"=>
		niosstate<=AVA_DATA_A_JTAG;
			
		when x"b"=>
		niosstate<=AVA_DATA_B_JTAG;
			
		when x"c"=>
		niosstate<=AVA_WAIT_JTAG;
			
		when x"d"=>
		niosstate<=EXE_DATA_CONTROL;
			
		when others=>
		niosstate<=WAIT_COMMAND;
		end case;	
		


    when EXE_COMMAND	 =>	
	command<=rorcdata(0);
	--control_clk<=rorcdata(1);
	--control_fifo<=rorcdata(2);
	niosstate<=WAIT_COMMAND;
	niosstatecounter<=x"0";
	
	when EXE_COMMAND_ADDR	 =>	
	command_addr<=rorcdata(0);
	niosstate<=WAIT_COMMAND;
	niosstatecounter<=x"1";
    
	when EXE_ADDR	 =>	
	send_addr<=rorcdata(6 downto 0);
	niosstate<=WAIT_COMMAND;
	niosstatecounter<=x"2";
		
    when EXE_DATA_A	 =>	
	if data_control=x"00" then
	rorcdataA<=rorcdata(15 downto 0);
	niosstate<=WAIT_COMMAND;
	end if;
	
	if data_control=x"01" then
	data_num(15 downto 0)<=rorcdata(15 downto 0);
	niosstate<=WAIT_COMMAND;
	end if;
	
	if data_control=x"02" then
	event_num<=rorcdata(15 downto 0);
	niosstate<=WAIT_COMMAND;
	end if;
	
	if data_control=x"03" then
	control_clk<=rorcdata(0);
	niosstate<=WAIT_COMMAND;
	end if;
		
	if data_control=x"04" then
	control_fifo<=rorcdata(0);
	niosstate<=WAIT_COMMAND;
	end if;	
	
	niosstatecounter<=x"3";	
	
		
    when EXE_DATA_B	 =>	

	if data_control=x"00" then
	fifo_data1<=rorcdata(15);
	fifo_data2<=rorcdata(14 downto 0) & rorcdataA(15 downto 11);
	fifo_data3<=rorcdataA(10);
	fifo_data4<=rorcdataA(9);
	fifo_data5<=rorcdataA(8 downto 0);
	nios_req<='0';
	niosstate<=EXE_WRITE;
	end if;

	if data_control=x"01" then
	data_num(31 downto 16)<=rorcdata(15 downto 0);
	niosstate<=WAIT_COMMAND;
	end if;
	

	
	niosstatecounter<=x"4";

	when EXE_DATA_CONTROL	 =>	
	data_control<=rorcdata(7 downto 0);
	niosstate<=WAIT_COMMAND;
	niosstatecounter<=x"d";

    when EXE_WRITE	 =>	
	if nios_req='0' then
		fifo_data1<=rorcdata(15);
		fifo_data2<=rorcdata(14 downto 0) & rorcdataA(15 downto 11);
		fifo_data3<=rorcdataA(10);
		fifo_data4<=rorcdataA(9);
		fifo_data5<=rorcdataA(8 downto 0);
		nios_req<='1';
	else
		nios_req<='0';
		niosstate<=WAIT_COMMAND;	
	end if;
	niosstatecounter<="1000";

    when EXE_RESET	 =>	
	if reset_n='1' and CONV_INTEGER(reset_counter)=0 then
		reset_n<='0';
		reset_counter<=x"0";
	else
	reset_counter<=reset_counter+1;
	if CONV_INTEGER(reset_counter)>5 then
		reset_n<='1';
	end if;
	
	if CONV_INTEGER(reset_counter)>12 then
		niosstate<=WAIT_COMMAND;
		reset_counter<=x"0";
	end if;
	
	end if;
	niosstatecounter<=x"5";

	when EXE_TRIGGER	 =>	
	if trg_come='0' then
		trg_come<='1';
	else
		trg_come<='0';
		niosstate<=WAIT_COMMAND;
	end if;
	niosstatecounter<=x"6";
	
	when EXE_READ	 =>	
	if wrreq_ddl='0' then
		case rorcdata(1 downto 0) is
		when "00"=>
		data_in_ddl<=data_in;
	
		when "01"=>
		data_in_ddl<=states;
		
		when "10"=>
		data_in_ddl<=block_data;
	
		when "11"=>
		data_in_ddl<=avalon_readdata;
--		data_in_ddl<=x"aaaaaaaa";
		end case;
		
		wrreq_ddl<='1';
	else
		wrreq_ddl<='0';
		niosstate<=WAIT_COMMAND;
	end if;
	niosstatecounter<=x"7";

	when AVA_CS_JTAG=>
	niosstatecounter<=x"8";
	if avalon_read='1' then
		--avalon_readdata_back<=avalon_readdata;
	end if;
	
	if avalon_chipselect='1' then
		avalon_chipselect<='0';
		niosstate<=WAIT_COMMAND;
	else
		avalon_chipselect<='1';
	end if;
	
	
	when AVA_ADDR_JTAG=>
	--niosstatecounter<="0010";
	niosstatecounter<=x"9";
	avalon_address<=rorcdata(1 downto 0);
--	if rorcdata(2)='1' then
--		avalon_write<='1';
--		avalon_read<='0';
--	else
--		avalon_write<='0';
--		avalon_read<='1';
--	end if;
	avalon_write <= rorcdata(2);
	avalon_read  <= not rorcdata(2);
		niosstate<=WAIT_COMMAND;
	
	
	when AVA_DATA_A_JTAG=>
	niosstatecounter<=x"A";
--	niosstatecounter<="0001";
	avalon_writedata(15 downto 0)<=rorcdata(15 downto 0);
	--avalon_buffer(15 downto 0)<=rorcdata(15 downto 0);
	niosstate<=WAIT_COMMAND;
	

	when AVA_DATA_B_JTAG=>
	niosstatecounter<=x"B";
	avalon_writedata(31 downto 16)<=rorcdata(15 downto 0);
	--avalon_buffer(31 downto 16)<=rorcdata(15 downto 0);
	niosstate<=WAIT_COMMAND;
	

	when AVA_WAIT_JTAG=>
	niosstatecounter<=x"C";
		jtagcounter<=jtagcounter+1;
	avalon_readdata_back<=avalon_readdata;
	if avalon_chipselect='1' and avalon_address="11" and avalon_read='1' and  CONV_INTEGER(jtagcounter)>400 then
		if avalon_readdata_back(15 downto 0)=rorcdata(15 downto 0) then
			avalon_chipselect<='0';
			niosstate<=WAIT_COMMAND;
		end if;
		if CONV_INTEGER(jtagcounter)>500000 then
			avalon_chipselect<='0';
			niosstate<=WAIT_COMMAND;
		end if;
	else
		avalon_chipselect<='1';
		avalon_address<="11";
		avalon_read<='1';
		avalon_write<='0';
	end if;
	
	when others=>
	niosstate<=WAIT_COMMAND;
	end case;
end if;
end process;
-------------------------------------------------------------------------nios
----------------------------------------------------------------------get data from DC
process (RHIC_CLK)
begin
if RHIC_CLK='0' and RHIC_CLK'event then
	if control_fifo='1' and command_addr='1' and eventstart='1' then
--	if control_fifo='1' and command_addr='1'  then
		frame_data<=fifo_data1 & fifo_data2 & fifo_data3 & fifo_data4 & fifo_data5;
		frame_wrreq<=fifo_read_ready;
		
--		if frame_data(0)='0' then
--		frame_data<=x"55555555";
--		else
--		frame_data<=x"aaaaaaaa";
--		end if;
--		frame_wrreq<='1';
	else
		frame_wrreq<='0';
	end if;
end if;
end process;
--frame_wrreq<=fifo_read_ready when control_fifo='1' and command_addr='1' else '0';

----------------------------------------------------------------------get data from DC
process (ava_clk)
begin
if ava_clk='0' and ava_clk'event then

	wrreqdelay1<=Cluster_FIFO_Data_Valid and req_reg;
	wrreqdelay2<=wrreqdelay1;
	
	if control_fifo='0' then
		data_in<=fifo_data1 & fifo_data2 & fifo_data3 & fifo_data4 & fifo_data5;
--data_in<=x"00000080";
	end if;
	if command='0'  then
		if req_reg='1' then
				wrreq<='1';
		else
				wrreq<='0';
		end if;
	end if;
end if;
end process;

--------------------------------------------------------------------------control register
process (ava_clk)
begin
if reset_n='0' then
elsif ava_clk='1' and ava_clk'event then


	if readflag='1' and fifo_read_ENA='1' then
		req_reg<='1';
	else 
		req_reg<='0';
	end if;

	if command='0' then 
		wrreq_bus<=wrreqdelay2 ;
		data_in_bus<=data_in;
	else
		wrreq_bus<=wrreq_ddl;
		data_in_bus<=data_in_ddl;
	end if;
	
	if rdempty='1' or CONV_INTEGER(fifoused)<800 then
		readflag<='1' ;
	elsif fifo_read_ENA='0' and CONV_INTEGER(fifoused)>14700  then
	readflag<='0' ;
	end if;

	if command='0' then
		fifo_read_REQ<=	req_reg;
	elsif ava_addr="000100"  and niosstate=WAIT_COMMAND then
		fifo_read_REQ<=	ava_cs;
	elsif niosstate=EXE_WRITE then
		fifo_read_REQ<=	nios_req;
	else
		fifo_read_REQ<=	'0';
	end if;

	states<="0000000" & control_fifo & fiDIR & fiBEN_N & fiLF_N &  re_addr  & send_addr  & command & command_addr & readflag & req_reg & fifo_read_ENA	& rdempty & wrreq  ;	
--	states<="00000000" & fiDIR & fiBEN_N & fiLF_N & "000" & timescounter  & "000" & statecounter & command & command_addr & readflag & req_reg & fifo_read_ENA	& rdempty & wrreq  ;	
	block_data<=x"00" & "00" & fifoused & niosstatecounter & statecounter;
	--show_fifoused<=fifoused;
		
	if command_addr='1' then		
		fifo_addr<=send_addr;
	else
		fifo_addr<=re_addr;
	end if;

end if;
end process;






--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww

--iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii
inaclr<=not reset_n;

infifo_i: infifo
	PORT map
	(
		data		=>indata_in,
		wrreq		=>inwrreq,
		rdreq		=>inrdreq,
		rdclk		=>not ava_clk,
		wrclk		=>not ava_clk,
		aclr		=>inaclr,
		q			=>inq,
		rdempty		=>inrdempty,
		wrfull		=>inwrfull
	);

frame_fifo: framefifo
	PORT map
	(
		data		=>frame_data,
--		wrreq		=>'0',
		wrreq		=>frame_wrreq,
		rdreq		=>frame_rdreq,
		rdclk		=>ava_clk,
		wrclk		=>RHIC_CLK,
		aclr		=>not reset_n,
		q			=>frame_q,
		rdempty		=>frame_rdempty,
		wrfull		=>frame_wrfull
	);

myfifo_i: myfifo											  -- instantiate my fifo
	PORT map
	(
		data		=>data_in_bus,
		wrreq		=>wrreq_bus,
		rdreq		=>rdreq,
		rdclk		=> ava_clk,
		wrclk		=> ava_clk,
		aclr		=>not reset_n,
		q			=>q,
		rdempty		=>rdempty,
		wrfull		=>wrfull,
		wrusedw		=>fifoused
	);

readdeligater_i: readdeligater

port map(
		reset_n => reset_n ,


	clk =>  ava_clk,
	
-- interface to the ClusterFifoModules
	cf_address => re_addr,
	cf_read_ready =>fifo_read_ready,
	cf_read_enable =>fifo_read_ENA,
--new control line 
   	cf_control => '0',
	nios_command=>command_addr & data_in(5 downto 0)

)	 ;

jtag_programmer_avalon_i: jtag_programmer_avalon 
	Port map ( 
		--Avalon signals
		avalon_clk 			=>not ava_clk,						
		avalon_reset 		=>not reset_n,					
		avalon_chipselect 	=>avalon_chipselect,					
		avalon_address 		=>avalon_address,	
		avalon_write 		=>avalon_write,					
		avalon_writedata 	=>avalon_writedata, 
		avalon_read 		=>avalon_read,					
		avalon_readdata 	=>avalon_readdata,
		
		-- JTAG outputs
		TMS_out =>TMS_out,
		TDI_out =>TDI_out,
		TCK_out =>TCK_out,
		TDO_in 	=>TDO_in,
		RSTB_out=>RSTB_jtag
		);
--iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii
END a;




