--------------------------------------------------------------------------------
--	 module name :ClusterFifos
--function: waiting for a trigger, after trigger comes, prepares a data head when waiting the 
--real data come, then accept the real data until it ends and add a end token at the end of the data. 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-- in ARCHITECTURE 
-- between component          ccccccccccccccccccccccccccccccccccccccccccccccccccc
-- between signal declaration sssssssssssssssssssssssssssssssssssssssssssssssssss
-- between wire connent       wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
-- between process            ppppppppppppppppppppppppppppppppppppppppppppppppppp
-- between instantiation	   iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii

entity ClusterFifos is
generic (
	ADDRSIZE 	: integer  :=9 ;
	DATASIZE 	: integer  :=20;
	WAIT_WINDOW_LENGTH 	: integer  :=6;
	CLUSTER_WINDOW_LENGTH	:	integer :=10;
	FIFO_LENGTH	:	integer :=200;
	CHIP_ID	:	integer :=10
	 );
port (
 	reset_n : in std_logic ;

-- clocks
	pix_clk 	: in std_logic ; -- same clock that moves pixels through the cluster finder
--	dbdb_clk	: in std_lobic ; -- read end clk, Daughter Board Data Bus

-- trigger data
	trg_word: in std_logic_vector(19 downto 0) ;															
															-- this gets loaded into the beginning of each
															-- event block 
--	chip_id : in std_logic_vector(11 downto 0) ; -- ladder(11 downto 6) & chip(5 downto 0)
												-- this also gets loaded at the beginning of each block
															
-- input data and interface control
	data_in: in std_logic_vector(17 downto 0) ;															
	got_cluster	: in std_logic;  --from cluster finder, found one
	start			: in std_logic; -- contains TRG command, DAQ command, token
--	got_cluster : in std_logic; -- from cluster finder, found one
	idle_flag	:out std_logic; --true when module is in idle waiting for trigger
--	do_aborts	:	in std_logic; -- a control bit to enable module to respond to trigger aborts
	
-- output data and interface control
--	address		: in  std_logic_vector(5 downto 0);-- each CFM for the ladder has unique address
	data_out			: out std_logic_vector(19 downto 0); -- output data, tri stated.
	read_ready	: out std_logic; --true when module is ready to be read out, tri stated.
	rdreq	:	in std_logic; -- true causes fifo to clock data, q, out until empty (when this CFM addressed)

----new 
	read_clk 		: in std_logic;
	chip_address	: in std_logic_vector(5 downto 0)
	
		);
end entity ClusterFifos;

ARCHITECTURE a of ClusterFifos is


--cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
component myfifo
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (19 DOWNTO 0);
		wrreq		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		rdclk		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		aclr		: IN STD_LOGIC  := '0';
		q			: OUT STD_LOGIC_VECTOR (19 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;
--ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
--sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
type stype is (WAIT_TRG, WAIT_WINDOW,WRITTING,WAIT_READ);
type rtype is (IDLE,READING);
signal state: stype:=WAIT_TRG;




--signal write_distribute : STD_LOGIC_vector(4 downto 0):="00000";
signal write_distribute : STD_LOGIC_vector(0 downto 0):="0";
--signal get_distribute 	: STD_LOGIC_vector(4 downto 0):="00000";
signal get_distribute 	: STD_LOGIC_vector(0 downto 0):="0";
-- we use write_distribute and get_distribute as a trigger token before and after trigger comeing


signal write_data 			: STD_LOGIC_vector(DATASIZE-1 downto 0);
signal read_data 			: STD_LOGIC_vector(DATASIZE-1 downto 0);
signal write_pre : STD_LOGIC:='0';	 -- the pre request of write, 
signal read_req : STD_LOGIC:='0';	 -- read request signal
signal write_req : STD_LOGIC:='0';	 -- write request signal

signal w_write_over : STD_LOGIC:='0';	--writing being over signal 
signal r_write_over : STD_LOGIC:='0';	--reading being over signal
signal write_over : STD_LOGIC:='0';		--exchange data between write and read processes


signal window_counter : integer:=0;		--counts the length of waitting window
signal write_counter : integer:=0;		--counts the number of data from cluster-finding  despite	there is a cluster
--signal written_counter : integer:=0;	--counts the number of data written in fifo
--signal read_counter : integer:=0;		--counts the number of data read from fifo
--signal sum_counter : integer:=0;			--counts the number of data in fifo.

signal rdempty : STD_LOGIC:='0';				--no use
signal wrfull : STD_LOGIC:='0';
--ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss




begin
--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
read_ready<=  write_over  when CONV_INTEGER(chip_address)=CHIP_ID else	
			'Z';		 -- if data writing is over, set write_over.

read_req<=rdreq and write_over when CONV_INTEGER(chip_address)=CHIP_ID else	
			'0'; --if data writing is over, reading is permitted


write_over<= w_write_over xor r_write_over;
--f1<=f2 xor f3;
-- sum_counter<=	written_counter-read_counter;	 --sun_counter counts data number in fifo

--  get_distribute<= write_distribute+1 when write_distribute(0)='0' else
--						 write_distribute+2 when write_distribute(1)='0' else
--						 write_distribute+4 when write_distribute(2)='0' else
--						 write_distribute+8 when write_distribute(3)='0' else
--						 write_distribute+16 when write_distribute(4)='0' else
--						 write_distribute;

get_distribute<= write_distribute+1 when write_distribute(0)='0' else --we use write_distribute to indicate tirgger's coming.
																							-- if a trigger comes write_distribute will be set	  
					 write_distribute;

write_data<="00000000000000000111"  when	window_counter=1 else  --when we waiting in window, we can put data head in fifo
				trg_word					   when	window_counter=2 else  --when we waiting in window, we can put data head in fifo
 CONV_STD_LOGIC_VECTOR(CHIP_ID,20)  when window_counter=3	 else  --when we waiting in window, we can put data head in fifo
				"11000000000001011111"  when	write_counter=CLUSTER_WINDOW_LENGTH else  --when we waiting in window, we can put data head in fifo
				"00" & data_in			when write_distribute(0)='1';
data_out<=read_data  when 	write_over='1'  and CONV_INTEGER(chip_address)=CHIP_ID else	--when data writing is over, reading is requested,	data_out is connected.
				"ZZZZZZZZZZZZZZZZZZZZ";
write_req<='1' when 	(window_counter>0 and window_counter<=3) or  (write_counter=CLUSTER_WINDOW_LENGTH and state=WRITTING)  else --when we are setting data head or writing data into fifo, writing is permitted
             write_pre and got_cluster when write_distribute(0)='1' else 
			'0';										--write_req is the write request signal in fifo
									--write_req is the write request signal in fifo

idle_flag<= '1' when state=WAIT_TRG else					--  system idle when it waiting for a trigger
			'0';
--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
----------------------------------------------------------------------------------------------
process(pix_clk,reset_n)						  --write process
begin
if  reset_n='0' then
	  write_counter<=0;
--	  written_counter<=0;
	  window_counter<=0;
  	 write_pre<='0';									--forbid data writing.
  	 w_write_over<='0';									--forbid data writing.
	 state <= WAIT_TRG;
	write_distribute<="0";
 else


if pix_clk='1' and pix_clk'event  then

if  start='1' then--and start'last_value='0' then		 --a trigger comes, set a bit of write_distribute
  write_distribute<=get_distribute;

end if;

if state=WAIT_READ and rdempty='1' and start='0' then	--when reading is over, zero the bit of write_distribute
write_distribute(0)<='0';
end if;
case state is
    when WAIT_TRG	 =>								  -- waitting for a trigger, then switch to WAIT_WINDOW
--		reset <='1';
		if write_distribute(0)='1' then			  --if a trigger arrives , switch to WAIT_WINDOW
--		reset <='0';
		 window_counter<=0;							 
		 write_pre<='0';									--forbid data writing.
		 state<= WAIT_WINDOW;
		 end if;
    when WAIT_WINDOW	 =>


 		if window_counter=WAIT_WINDOW_LENGTH-1 then		  -- WAIT_WINDOW is over , turn to WRITTING
		write_counter<=0;									  -- in WAIT_WINDOW, we set data head, so we 
		 write_pre<='1' ;									 --should add corresponding value in written_counter
	--	written_counter<=written_counter+1;
		 state<= WRITTING;
		 else
		 window_counter<=window_counter+1; 
		 end if;	
	 when WRITTING	 =>


		 	if write_counter=CLUSTER_WINDOW_LENGTH then			--	WRITTING is over , turn to WAIT_READ
			write_pre<='0';
		 w_write_over<=not w_write_over;				-- tell reading process, you can read
		 state<= WAIT_READ;
		 else 
		 if got_cluster='1' then
	-- 	 written_counter<=written_counter+1;
	 	 write_counter<=write_counter+1;				-- for test
		end if;
--	 	 write_counter<=write_counter+1;				-- real counter
		
			end if;
    when WAIT_READ =>
		write_counter<=0;
	 if rdempty='1' and write_distribute(0)='0'  then	-- if reading is over and write_distribute is zeroed
		 state<= WAIT_TRG;											--got_cluster a new process.
		 end if;
end case;

end if;

end if;
end process;
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
process(read_clk,reset_n)										  --read process
begin 

if  reset_n='0' then
--	  read_counter<=0;
  	 r_write_over<='0';								

 else

if  read_clk='1'and read_clk'event then

		 	if rdempty='1' and write_over='1' then	 -- is readong is over ,tell 
		r_write_over<=not r_write_over;					 --trigger process to zero  write_distribute

--		 	elsif  write_over='1'and rdreq='1' then	 -- reading data
--	 	 read_counter<=read_counter+1;
		end if;



end if;


end if;
end process;
---------------------------------------------------------------------------------------------

--iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii
myfifo_i: myfifo											  -- instantiate my fifo
	PORT map
	(
		data		=>write_data,
		wrreq		=>write_req,
		rdreq		=>read_req,
		rdclk		=>read_clk,
		wrclk		=>pix_clk,
		aclr		=>not reset_n,
		q			=>read_data,
		rdempty		=>rdempty,
		wrfull		=>wrfull
	);
--iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii


--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww

--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
end architecture;
