--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    11:16:14 05/15/06
-- Design Name:    
-- Module Name:    myconfig - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity myconfig is
generic (
	SELECTION_D 	: integer  :=9 ; -- chip select command
	CLEANUP_D  		: integer  :=20; -- clean up command
	READPIXCLE_D 	: integer  :=6;  --read command
	CONFIGSRAM_D	:	integer :=10; --write circular buffer command
	MAPSIZE_D	:	integer :=50;
	BLOCKSIZE_D	:	integer :=10;
	OFFSET_D		:	integer :=4	;
	INVALID_D		:	integer :=21
	
	 );
    Port ( clk : in std_logic;
           reset_n : in std_logic;
           from_adc : in std_logic_vector(11 downto 0);
           from_addr : in std_logic_vector(17 downto 0);
           from_config_in : in std_logic_vector(19 downto 0);
           to_sram_addr : out std_logic_vector(19 downto 0);
           to_sram_data : out std_logic_vector(31 downto 0);
           to_sram_write : out std_logic;
           to_config_in_rreq : out std_logic;
           from_config_in_empty : in std_logic;
           to_config_out : out std_logic_vector(19 downto 0);
           to_config_out_wreq : out std_logic;
           from_config_out_full : in std_logic;
			  to_adc: out std_logic_vector(1 downto 0)
			  );
end myconfig;

architecture Behavioral of myconfig is
--cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
component myfifo
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (19 DOWNTO 0);
		wrreq		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		rdclk		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		aclr		: IN STD_LOGIC  := '0';
		q			: OUT STD_LOGIC_VECTOR (19 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;
--ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
--sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
type stype is (WAIT_COMMAND, CLEANUP, CHIP_SELECT,READING);
type com_type is (IDLE,READ_IN,SEND_OUT);
type select_type is (SELECT_IDLE,SELECT_READ_IN,SELECT_SEND_OUT);
type read_type is (WAIT_START,READ_START,READ_OVER);
type write_type is (WRITE_IDLE,WRITE_READ_IN,WRITE_SEND_OUT);


signal state,next_state: stype:=WAIT_COMMAND;
signal com_state,next_com_state: com_type:=IDLE;
signal sec_state,next_sec_state: select_type:=SELECT_IDLE;
signal red_state,next_red_state: read_type:=WAIT_START;
signal w_state,next_w_state: write_type:=WRITE_IDLE;

signal  data_config_in :  std_logic_vector(19 downto 0);
signal  data_config_out :  std_logic_vector(19 downto 0);
signal  chip_selecting:  std_logic_vector(1 downto 0):="00";
signal  sub_chip_selecting:  std_logic_vector(5 downto 0):="000111";
signal  data_from_addr:  std_logic_vector(17 downto 0);
signal  map_length:  std_logic_vector(5 downto 0):="000000";
signal  write_map_address:  std_logic_vector(5 downto 0):="000000";
signal  write_map_data   :  std_logic_vector(5 downto 0):="000000";

signal  block_length:  std_logic_vector(5 downto 0):="000000";
		  
--sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss

begin
--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
to_config_out<=data_config_in 			   when com_state=SEND_OUT       				   else
               "00000000000000000011"     when  sec_state=SELECT_SEND_OUT					 else
					"00000000000000000111"     when state=CLEANUP 				           else
					"11000000000000000000"		when block_length=BLOCKSIZE_D	 		   else
					"00000000"&	from_adc			when state=READING					   else
					(others=>'Z')
					;
data_from_addr<=from_addr-4;
to_adc<=chip_selecting;
--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
----------------------------------------------------------------------------------------------
process(clk,reset_n)						  --write process
begin
if  reset_n='0' then
 state<=WAIT_COMMAND;
 next_state<= WAIT_COMMAND;
com_state<=IDLE;
next_com_state<=IDLE;
sec_state<=SELECT_IDLE;
next_sec_state<=SELECT_IDLE;
red_state<=WAIT_START;
next_red_state<=WAIT_START;

 else
if clk='1' and clk'event  then
state<=next_state;
com_state<=next_com_state;
sec_state<=next_sec_state;
red_state<=next_red_state;
w_state<=next_w_state;
 case state is
    when WAIT_COMMAND =>
		case 	com_state is
		when  IDLE =>
		 		 to_config_out_wreq<='0';
		 		 to_config_in_rreq<='0';
		 if from_config_in_empty='0' and next_state=WAIT_COMMAND  then
		 next_com_state<= READ_IN;
		 end if;
		when  READ_IN =>
		 to_config_in_rreq<='1';
		data_config_in<=from_config_in;
		 next_com_state<= SEND_OUT;
		when  SEND_OUT=>
 		 to_config_in_rreq<='0';

	    if data_config_in=conv_std_logic_vector(SELECTION_D,20)  then
  		  if	next_com_state/= com_state then
			   		 to_config_out_wreq<='0'; 
			else
					 to_config_out_wreq<='1';
		  next_com_state<=IDLE;
		  next_state<=  CHIP_SELECT;

		  end if;
		 elsif data_config_in=conv_std_logic_vector(CLEANUP_D,20)  then
		  if	next_com_state/= com_state then
			   		 to_config_out_wreq<='0';
			else
		 to_sram_write<='1';
   		 to_config_out_wreq<='1';
		  next_com_state<=IDLE;
		  next_state<=  CLEANUP;
		map_length<=(others=>'0');

		  end if;
	    elsif data_config_in=conv_std_logic_vector(READPIXCLE_D,20)  then
 		  if	next_com_state/= com_state then
			   		 to_config_out_wreq<='0'; 
			else
		  to_config_out_wreq<='1';
		  next_com_state<=IDLE;
		  next_state<=  READING;
		  end if;
		  else
		  next_com_state<=IDLE;
		  end if;		
		  end case;

   when CHIP_SELECT =>
		case sec_state is
		when SELECT_IDLE=>

		 		if from_config_in_empty='0' and next_state=CHIP_SELECT  then
		 		next_sec_state<= SELECT_READ_IN;
		 		end if;
		 when SELECT_READ_IN=>
				 to_config_in_rreq<='1';
				data_config_in<=from_config_in;
		 		next_sec_state<= SELECT_SEND_OUT;
		 when SELECT_SEND_OUT=>
				  if	next_sec_state/= sec_state then
			   		 to_config_out_wreq<='0';	
			else
				 to_config_in_rreq<='0';
				 if    data_config_in(19 downto 17)="001" then 
				 chip_selecting<=data_config_in(1 downto 0);
				 				 to_config_out_wreq<='1';
				 elsif data_config_in(19 downto 17)="010" then
				 sub_chip_selecting<=data_config_in(5 downto 0);
								 to_config_out_wreq<='1';
				elsif data_config_in(19 downto 18)="10" then
				to_sram_addr<="00000000000000"&data_config_in(5 downto 0);
				 to_sram_data<="10000000000000000000000000000000";
				 to_sram_write<='1';
				 to_config_out_wreq<='1';
				 end if;
				  next_sec_state<=SELECT_IDLE;
				  next_state<=  WAIT_COMMAND;
				  end if;

		end case;
   when CLEANUP =>
		if	next_state/= state then
		 to_config_out_wreq<='0';	
		elsif CONV_INTEGER(map_length)=MAPSIZE_D then
			 next_state<=  WAIT_COMMAND;
 		to_sram_write<='0';
		 to_config_out_wreq<='1';
		else
		to_sram_addr(5 downto 0)<=map_length;
		to_sram_data<=(others=>'0');
		map_length<=map_length+1;
		end if;


   when READING =>
		 case red_state is
		 when  WAIT_START=>
		  		 to_config_out_wreq<='0';
		 		 to_config_in_rreq<='0';

			 if data_from_addr="000000000000"&sub_chip_selecting then
				 next_red_state<=READ_START;
				block_length<=(others=>'0');
			  end if;
		 when  READ_START=>
		 	if	next_red_state/= red_state then
			   		 to_config_out_wreq<='0';
			elsif  CONV_INTEGER(block_length)=BLOCKSIZE_D	then
			  next_red_state<=WAIT_START;
			  next_state<=  WAIT_COMMAND;
			elsif from_config_out_full='1' then
			  next_red_state<=READ_OVER;
			else
						 to_config_out_wreq<='1';
						  block_length<=block_length+1;
		  			end if;
 		 when  READ_OVER=>
			  block_length<=(others=>'0');
			  if from_config_out_full='0' then
				  next_red_state<=WAIT_START;
				  next_state<=  WAIT_COMMAND;
			  	  end if;
			end case;
end case;

end if;	 

end if;
end process;
----------------------------------------------------------------------------------------------

end Behavioral;
