LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

ENTITY user IS
	PORT
	(  
  
--	Dev. Kit. ******************
		--		resets
		RESETn			: INOUT STD_LOGIC ;	-- push button RESET, on PIN
		CLEARn			: IN STD_LOGIC ;	-- push button CLEAR
		FAST_CLOCK		: out std_logic ; -- for tests!
--		SDROM1_CLOCK		: out std_logic ; -- for tests!
--		SDROM2_CLOCK		: out std_logic ; -- for tests!
		-- config stuff
		PLD_PGM			: OUT STD_LOGIC_VECTOR(2 downto 0) ;	-- config page
		
		--		clocks
		SYSCLK			: IN  STD_LOGIC;	-- 32.768 MHz Dev. Kit. crystal
		NIOS_CLK_e		: OUT STD_LOGIC ;	-- for debugging

		--	UARTs		
		UART_TXD		: OUT STD_LOGIC ;
		UART_RXD		: IN STD_LOGIC ;		
		--	Switches
		switch			: IN STD_LOGIC_VECTOR(3 downto 0) ;
		--	push buttons
		button			: IN STD_LOGIC_VECTOR(1 downto 0) ;

-- SRAM
		mem_dq			: INOUT	STD_LOGIC_VECTOR(31 downto 0) ;
		mem_a			: OUT STD_LOGIC_VECTOR(20 downto 0) ;
		sram_a1			: OUT std_logic ;
		sram_be_n		: OUT STD_LOGIC_VECTOR(3 downto 0) := (OTHERS => 'Z');
		sram_cs0_n		: OUT std_logic ;
		sram_cs1_n		: OUT std_logic ;
		sram_we_n		: OUT std_logic := 'Z' ;
		sram_oe_n		: OUT std_logic := 'Z' ;
-- FLASH
		flash_mem_rp_n	: out std_logic ;
		flash_oe_n		: out std_logic ;
		flash_we_n		: out std_logic ;
		flash_byte_n	: out std_logic ;
		flash_ce_n		: out std_logic ;
-- motherboar

		--leo			: INout	STD_LOGIC_VECTOR(72 downto 0); -- the data I.E. the address of cluster center
		--
		leo		: INout	STD_LOGIC_VECTOR(68 downto 0);
		--leo		: INout	STD_LOGIC_VECTOR(59 downto 0);
		TMS_out : out std_logic;
		TDI_out : out std_logic;
		TCK_out : out std_logic;
		TDO_in 	: in std_logic;
		RSTB_jtag: out std_logic;

--ddl
		fbD			: INOUT	STD_LOGIC_VECTOR(31 downto 0) ;
		fbCTRL_N	: INOUT	STD_LOGIC ;
		fbTEN_N		: INOUT	STD_LOGIC ;

		fiDIR		: IN STD_LOGIC ;
		fiBEN_N		: IN STD_LOGIC ;
		fiLF_N		: IN STD_LOGIC ;

		foBSY_N		: OUT STD_LOGIC ;
		foCLK		: OUT STD_LOGIC ;
--led
        Michal		: out STD_LOGIC_VECTOR(1 downto 0)

	);
END user;

ARCHITECTURE a OF user IS
--ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
component nios is 
           port (
                        -- 1) global signals:
                 signal clk : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;

              -- the_J23_SW
                 signal in_port_to_the_J23_SW : IN STD_LOGIC;

              -- the_dev_uart
                 signal rxd_to_the_dev_uart : IN STD_LOGIC;
                 signal txd_from_the_dev_uart : OUT STD_LOGIC;

              -- the_test_0_test
                 signal addr_to_the_test_0 : OUT STD_LOGIC_VECTOR (5 DOWNTO 0);
                 signal cs_to_the_test_0 : OUT STD_LOGIC;
                 signal readdata_from_the_test_0 : IN STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal wreq_from_the_test_0 : IN STD_LOGIC;
                 signal write_n_to_the_test_0 : OUT STD_LOGIC;
                 signal writedata_to_the_test_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);

              -- the_tri_state_bridge_avalon_slave
                 signal be_n_to_the_SRAM : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
                 signal read_n_to_the_SRAM : OUT STD_LOGIC;
                 signal select0_n_to_the_SRAM : OUT STD_LOGIC;
                 signal select1_n_to_the_SRAM : OUT STD_LOGIC;
                 signal select_n_to_the_epc : OUT STD_LOGIC;
                 signal tri_state_bridge_address : OUT STD_LOGIC_VECTOR (20 DOWNTO 0);
                 signal tri_state_bridge_data : INOUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal tri_state_bridge_readn : OUT STD_LOGIC;
                 signal write_n_to_the_SRAM : OUT STD_LOGIC;
                 signal write_n_to_the_epc : OUT STD_LOGIC
                 );
end component nios;

component pll_main
	PORT
	(
		inclk0		: IN STD_LOGIC  := '0';
		e2		: OUT STD_LOGIC ;
		c0		: OUT STD_LOGIC ;
		c1		: OUT STD_LOGIC ;
		c2		: OUT STD_LOGIC ;
		c3		: OUT STD_LOGIC ;
		c4		: OUT STD_LOGIC ;
		c5		: OUT STD_LOGIC ;
		e0		: OUT STD_LOGIC ;
		e1		: OUT STD_LOGIC 
	);
end component;
component chinhvu
port(
--	Dev. Kit. ******************
		--		resets
		RHIC_CLK			: in  STD_LOGIC ;-- read request
		CLK_CONTROL			: out  STD_LOGIC ;-- read request
		fifo_read_REQ			: out  STD_LOGIC ;-- read request
		fifo_read_ready			: in  STD_LOGIC ;-- read request

		fifo_data1			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data2			: INout	STD_LOGIC_VECTOR(19 downto 0) ;-- the data I.E. the address of cluster center
		fifo_data3			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data4			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data5			: INout	STD_LOGIC_VECTOR(8 downto 0) ;-- the data I.E. the address of cluster center

		fifo_addr			: INout	STD_LOGIC_VECTOR(6 downto 0) ;-- the data I.E. the address of cluster center

		out_trg_come 		: out std_logic;
		out_reset_n 		: out std_logic;
		out_clock 			: out std_logic;
--ddl
		fbD			: INOUT	STD_LOGIC_VECTOR(31 downto 0) ;
		fbCTRL_N	: INOUT	STD_LOGIC ;
		fbTEN_N		: INOUT	STD_LOGIC ;

		fiDIR		: IN STD_LOGIC ;
		fiBEN_N		: IN STD_LOGIC ;
		fiLF_N		: IN STD_LOGIC ;

		foBSY_N		: OUT STD_LOGIC ;
		foCLK		: OUT STD_LOGIC;
			com		: OUT STD_LOGIC;
			test 	: out  std_logic_vector(6 downto 0) ;		

--avalon
		ava_clk : in std_logic ;
		ava_cs : in std_logic ;

		ava_wdata : in std_logic_vector(31 downto 0) ;
		ava_rdata : out std_logic_vector(31 downto 0) ;
	
		ava_write_n : in std_logic ;
		ava_addr : in std_logic_vector(5 downto 0) ;
	 
				
--
		TMS_out : out std_logic;
		TDI_out : out std_logic;
		TCK_out : out std_logic;
		TDO_in 	: in std_logic;
		RSTB_jtag: out std_logic;
		Cluster_FIFO_Data_Valid 	: in std_logic

);
end component;

--ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc




	

--sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
signal		RSTB_out		: 	STD_LOGIC ;
signal		mfbD			: 	STD_LOGIC_VECTOR(31 downto 0) ;
signal		mfbCTRL_N	: 	STD_LOGIC ;
signal		mfbTEN_N		: 	STD_LOGIC ;
signal		mfiDIR		:  STD_LOGIC ;
signal		mfiBEN_N		:  STD_LOGIC ;
signal		mfiLF_N		:  STD_LOGIC ;
signal		mfoBSY_N		:  STD_LOGIC ;
signal		mfoCLK		:  STD_LOGIC;

signal DDL_CLK, ALTRO_CLK, NIOS_CLK, RHIC_CLK, RHIC5_CLK : std_logic ;
signal SYS_RESETn : std_logic ;
signal SYSCLK_copy : std_logic ;
signal RHIC_clk_internal : std_logic ;
signal sclk1,sclk2,sclk3,sclk4: std_logic ;



-- external SRAM/FLASH helpers
signal data_bus : std_logic_vector(31 downto 0) ;
signal tri_bus : std_logic_vector(20 downto 0) ;
signal tri_bus_cs : std_logic ;
signal flash_we_n_int, flash_oe_n_int, flash_cs_n_int : std_logic ;

signal sram_cs0_n_int, sram_cs1_n_int : std_logic ;


--for test
signal pio : std_logic;
signal com : std_logic ;


signal test_rdata : std_logic_vector(31 downto 0) ;
signal test_wdata : std_logic_vector(31 downto 0) ;
signal test_addr : std_logic_vector(5 downto 0) ;
signal test_cs : std_logic ;
signal test_clk : std_logic ;
signal test_write_n : std_logic ;
signal test_wreq : std_logic ;
signal fifo_addr: std_logic_vector(6 downto 0) ;
signal	mytest :   std_logic_vector(6 downto 0) ;		

--sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss


BEGIN
--pio<='1' when test_addr="000001" and test_cs='1' and test_write_n='0' else
--  '0';
--pio<=leo(12);
pio<=com;


leo(19 downto 16)<=(others=>'Z');---LED
leo(22 downto 21)<=(others=>'Z');
--leo(25 downto 24)<=(others=>'Z');
leo(28 downto 27)<=(others=>'Z');
--leo(25)<='Z';

--leo(59 downto 29)<=data_bus(30 downto 0) ;
--leo(14)<=data_bus(31);

--leo(59 downto 29)<=(others=>'Z') ;
--leo(14)<='Z';



--leo(7 downto 1)<=fifo_addr;
--leo(7 downto 1)<=(others=>'Z');



-- DEV KIT stuff
-- always the same
--fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffixed
SYS_RESETn <= RESETn and CLEARn ;	-- any of the 2 goes low...
RESETn <= '0' when (button(0)='0') else 'Z' ; -- suicide

flash_ce_n <= '0' when (flash_cs_n_int='0') else 'Z' ;
sram_cs0_n <= sram_cs0_n_int ;
sram_cs1_n <= sram_cs1_n_int ;

-- careful to tri-state the FLASH/EPC lines!
tri_bus_cs <= not (flash_cs_n_int and sram_cs0_n_int and sram_cs1_n_int) ;

-- address bus tri-states...
mem_a <= tri_bus when(tri_bus_cs='1') else (others => 'Z') ;
sram_a1 <= tri_bus(19) when(tri_bus_cs='1') else 'Z';

-- misc EPC16 flash tri-states
flash_mem_rp_n <= RESETn when (flash_cs_n_int='0') else 'Z' ;
flash_byte_n <= '1' when (flash_cs_n_int='0') else 'Z' ;
flash_we_n <= flash_we_n_int when (flash_cs_n_int='0') else 'Z' ;
flash_oe_n <= flash_oe_n_int when (flash_cs_n_int='0') else 'Z' ;

--uuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuse component
nios_i: nios PORT MAP (
	clk => NIOS_CLK,
	reset_n => SYS_RESETn,
	in_port_to_the_J23_SW=>pio,
	rxd_to_the_dev_uart => UART_RXD,
	txd_from_the_dev_uart => UART_TXD,
--	rxd_to_the_rdo_uart => RDO_UART_RXD,
--	txd_from_the_rdo_uart => RDO_UART_TXD,

--		zs_addr_from_the_sdram_0=>addr_sdram_0,
--		zs_ba_from_the_sdram_0=>ba_sdram_0,
--		zs_cas_n_from_the_sdram_0=>cas_n_sdram_0,
--		zs_cke_from_the_sdram_0=>cke_sdram_0,
--		zs_cs_n_from_the_sdram_0=>cs_n_sdram_0,
--		zs_dq_to_and_from_the_sdram_0=>dq_sdram_0,
--		zs_dqm_from_the_sdram_0=>dqm_sdram_0,
--		zs_ras_n_from_the_sdram_0=>ras_n_sdram_0,
--		zs_we_n_from_the_sdram_0=>we_n_sdram_0,

           addr_to_the_test_0=>test_addr,
           cs_to_the_test_0 =>test_cs,
readdata_from_the_test_0=>test_rdata,
 wreq_from_the_test_0=>test_wreq,
write_n_to_the_test_0=>test_write_n,
writedata_to_the_test_0=>test_wdata,


    be_n_to_the_SRAM => sram_be_n,
    read_n_to_the_SRAM => sram_oe_n,
    select0_n_to_the_SRAM => sram_cs0_n_int,
    select1_n_to_the_SRAM => sram_cs1_n_int,
	select_n_to_the_epc => flash_cs_n_int,

    tri_state_bridge_address => tri_bus,
    tri_state_bridge_data => mem_dq(31 downto 0),
	tri_state_bridge_readn => flash_oe_n_int,
 


    write_n_to_the_SRAM => sram_we_n,


	write_n_to_the_epc => flash_we_n_int

	
);



--pll_main_i : pll_main PORT MAP (
--	inclk0 		=> SYSCLK,

--	e2 	=> NIOS_CLK_e,
--	e1  => sclk4,
--	e0 	=> sclk3,
	
--	c2 	=> test_clk,
--	c1 	=> ALTRO_CLK,
--	c0 	=> NIOS_CLK,

--	c5	=> fast_clock,
--	c4	=> RHIC_clk_internal,
--	c3 => SYSCLK_copy
--);

pll_main_i : pll_main PORT MAP (
	inclk0 		=> SYSCLK,

	e2 	=> sclk4,
	e1  => sclk3,
	e0 	=> NIOS_CLK_e,
	
	c2 	=> test_clk,
	c1 	=> ALTRO_CLK,
	c0 	=> NIOS_CLK,

	c5	=> fast_clock,
	c4	=> RHIC_clk_internal,
	c3 => SYSCLK_copy
);
chinhvu_i: chinhvu
	PORT map
	(
		leo(38),
		leo(39),
		leo(13),
		leo(8),
		leo(14),
		leo(59 downto 40),
		leo(23),
		leo(20),
		leo(37 downto 29),
		leo(7 downto 1),
--open,
		leo(9),
		leo(10),
		leo(11),
		fbD,	
		fbCTRL_N,
		fbTEN_N,

		fiDIR,
		fiBEN_N,
		fiLF_N,
		foBSY_N,
		foCLK,
		com,
		mytest,
		NIOS_CLK, --test_clk,
		test_cs,
		test_wdata,
		test_rdata,
		test_write_n,
		test_addr,
--		leo(72),
--		leo(71),
--		leo(69),
--		leo(70),
--		leo(72), --TMS_out
--		leo(71), --TDI_out
--		leo(69), --TCK_out
--		leo(70),	--TDO_in
--		leo(60) --RSTB_out
		TMS_out,
		TDI_out,
		TCK_out,
		TDO_in,
		--RSTB_jtag
		leo(12),
		leo(25)
	);
--process(test_clk)
--begin
--if  test_clk'event and test_clk='1' then
	--leo(72 downto 69)<=mytest(3 downto 0);
	--leo(68)<=test_clk;
	--leo(68 downto 65)<=mytest(3 downto 0);
	--leo(64 downto 62)<=mytest(6 downto 4);
	Michal(0)<=mytest(0);
	Michal(1)<='0';
--else
--	null;
--end if;
--end process;
--leo(67 downto 64)<=mytest(3 downto 0);
--leo(68)<=mytest(0);
--leo(4 downto 1)<=mytest(3 downto 0);
--uuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuse component





END a;
